#include <stdio.h>

#include "aplistitem.h"
#include <qfontmetrics.h>
#include <qapplication.h>
#include <qpainter.h>
#include "iwlib.h"

#define HEIGHT 48
#define LINE1 4
#define LINE2 28


QPixmap *ApListItem::wep = NULL;
QPixmap *ApListItem::wpa = NULL;
QPixmap *ApListItem::wpa2 = NULL;
QPixmap *ApListItem::w11b = NULL;
QPixmap *ApListItem::w11g = NULL;
QPixmap *ApListItem::anntena = NULL;

ApListItem::ApListItem(wireless_scan *scan, QListBox *listbox)
	: QListBoxItem(listbox)
{
	memset(&scan_info, 0, sizeof(scan_info));
	if(scan){
		scan_info = *scan;
		updateIcon();
	}

	if(!wep){
		wep = new QPixmap("/opt/QtPalmtop/share/wificonnect/wep.png");
		wpa = new QPixmap("/opt/QtPalmtop/share/wificonnect/wpa.png");
		wpa2 = new QPixmap("/opt/QtPalmtop/share/wificonnect/wpa2.png");
		w11b = new QPixmap("/opt/QtPalmtop/share/wificonnect/11b.png");
		w11g = new QPixmap("/opt/QtPalmtop/share/wificonnect/11g.png");
		anntena = new QPixmap("/opt/QtPalmtop/share/wificonnect/anntena.png");
	}
}

ApListItem::~ApListItem()
{
}

int ApListItem::height(const QListBox *box) const
{
//	int h = box ? box->fontMetrics().lineSpacing() + 2 : 0;
//	return QMAX( h, QApplication::globalStrut().height() );
	return HEIGHT;
}

int ApListItem::width(const QListBox *box) const
{
	int w = box ? box->fontMetrics().width( text() ) + 6 : 0;
	return QMAX( w, QApplication::globalStrut().width() );
}

void ApListItem::paint(QPainter *painter)
{
	QFont font(painter->font());
	QString str;
	int x;

	int iconpos = (HEIGHT - icon.height())/2;
	int textbase = HEIGHT - 2;

	/* 背景描画 */
	if(!selected() && (listBox()->index(this) & 1)){
		painter->fillRect(0, 0, listBox()->visibleWidth(), HEIGHT,
						  QBrush(QColor(180,180,180)));
	}

	/* アイコン描画 */
	painter->drawPixmap(iconpos, iconpos, icon);

	/* ESSID 描画 */
	if(scan_info.b.essid_on)
		str = scan_info.b.essid;
	else
		str = "<<unknown>>";

	QFontMetrics fm = painter->fontMetrics();

	painter->drawText(textbase, LINE1 + fm.ascent() + fm.leading()/2, str);
	painter->drawPixmap(iconpos, iconpos, icon);

	/* フォント変更 */
	font.setPixelSize(16);
	font.setWeight(QFont::Bold);
	painter->setFont(font);
	fm = painter->fontMetrics();

	/* 電波強度描画 */
	x = textbase;
	str = "%1\%";
	str = str.arg(scan_info.stats.qual.qual);
	painter->drawPixmap(x, LINE2, *anntena);
	x += anntena->width();
	painter->drawText(x, LINE2 + fm.ascent() + fm.leading()/2, str);

	/* WEP/WPA/WPA2 描画 */
	if(scan_info.b.has_key){
		x = listBox()->visibleWidth() - wpa2->width() - 3;
		if(scan_info.b.has_wpa2){
			painter->drawPixmap(x, LINE1+1, *wpa2);
			x -= wpa2->width();
		}
		if(scan_info.b.has_wpa){
			painter->drawPixmap(x, LINE1+1, *wpa);
			x -= wpa->width();
		}
		if(!scan_info.b.has_wpa && !scan_info.b.has_wpa2){
			painter->drawPixmap(x, LINE1+1, *wep);
			x -= wep->width();
		}
	}

	/* ビットレート＆アイコン */
	if(scan_info.has_maxbitrate){
		x = listBox()->visibleWidth() - 3;

		if(scan_info.maxbitrate.value > 11*1000000){
			x -= w11g->width();
			painter->drawPixmap(x, LINE2-1, *w11g);
		}
		x -= w11b->width();
		painter->drawPixmap(x, LINE2-1, *w11b);
		str = "%1Mbps";
		str = str.arg(scan_info.maxbitrate.value/1000000);

		x -= fm.width(str);
		painter->drawText(x, LINE2 - 1 + fm.ascent() + fm.leading()/2, str);
	}
}

void ApListItem::updateIcon()
{
	int s, l;
	QString path("/opt/QtPalmtop/pics144/netmon/WIFI%1.png");

	l = scan_info.stats.qual.qual;

	if(l < 20)
		s = 0;
	else if(l < 30)
		s = 1;
	else if(l < 60)
		s = 2;
	else
		s = 3;

	path = path.arg(s);
	icon.load(path);
}

void ApListItem::setScanInfo(const wireless_scan *scan)
{
	scan_info = *scan;
	updateIcon();
}
