/*
  mainwindow.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of WifiConnect.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "mainwindow.h"
#include <stdio.h>

#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qcombobox.h>

#include <stdio.h>
#include "wificonnect.h"

#include <qdatetime.h>
#include <qlabel.h>
#include "aplistbox.h"
#include "aplistitem.h"
#include "scanlist.h"
#include "scandialog.h"

MainWindow::MainWindow(QWidget *parent, const char *name, WFlags fl)
	: QWidget(parent, name, fl)
{
#ifndef QWS
	resize(640,640);
#endif

	hbox = new QHBoxLayout(this);
	QVBox *lvbox = new QVBox(this);
	hbox->addWidget(lvbox);
	rvbox = new QVBoxLayout(hbox);

	QHBox *ltop = new QHBox(lvbox);
	new QLabel("Device", ltop);
	device_combo = new QComboBox(ltop);
	device_combo->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));

	ap_list = new ApListBox(lvbox);
	ap_list->setText(QString::fromUtf8("無線LANカードが挿入されていません"));

	scan_button = new QPushButton("Scan", this);
	scan_button->setDisabled(true);
	connect_button = new QPushButton("Connect", this);
	connect_button->setDisabled(true);
	settings_button = new QPushButton("Settings", this);
	settings_button->setDisabled(true);
	QPushButton *exit_button = new QPushButton("Exit", this);

	rvbox->addWidget(scan_button);
	rvbox->addWidget(connect_button);
	rvbox->addWidget(settings_button);
	rvbox->addWidget(exit_button);

	dummy = new QWidget(this);
	dummy->setFixedWidth(0);
	rvbox->addWidget(dummy);

	connect(device_combo, SIGNAL(activated(const QString &)),
			this, SLOT(changeDevice(const QString &)));

	connect(scan_button, SIGNAL(clicked()), this, SLOT(clickScan()));
	connect(connect_button, SIGNAL(clicked()), this, SLOT(clickConnect()));
	connect(settings_button, SIGNAL(clicked()), this, SLOT(clickSettings()));
	connect(exit_button, SIGNAL(clicked()), this, SIGNAL(quit()));

	scan = new ScanList();

	updateComboBox();
#if 0
	wireless_scan scan;
	memset(&scan, 0, sizeof(scan));
	scan.b.essid_on = 1;
	strcpy(scan.b.essid, "hoge");
	scan.stats.qual.qual = 67;
	scan.has_maxbitrate = 1;
	scan.maxbitrate.value = 54*1000000;

	scan.b.has_key = 1;
	scan.b.has_wpa = 1;
	scan.b.has_wpa2 = 1;

	ApListItem *item = new ApListItem(&scan);
	ap_list->insertItem(item, -1);
#endif

	scan_button->setFocus();
}

MainWindow::~MainWindow()
{
}


static int handler(int skfd, char *ifname, char *args[], int count)
{
	QComboBox *combo = (QComboBox *)args[0];

	printf("fd:%d name:%s count:%d\n", skfd, ifname, count);
	combo->insertItem(ifname, -1);
	return 0;
}

void MainWindow::updateComboBox()
{
	device_combo->clear();

	char *arg[1];
	arg[0] = (char *)device_combo;
	iw_enum_devices(0, handler, arg, 1);

	if(device_combo->count() > 0){
		device_combo->setCurrentItem(0);
		scan->setDevice(device_combo->text(0).ascii());

		scan_button->setDisabled(false);
		ap_list->setText(QString::fromUtf8("アクセスポイントが無いかスキャンされていません"));
	}
}

void MainWindow::changeDevice(const QString &devname)
{
	scan->setDevice(devname.ascii());
}

void MainWindow::clickScan()
{
	ScanDialog dlg(scan, this);
	int ret = dlg.exec();
	ApList items = scan->getScanItems();

	ap_list->clear();

	int i;
	for(i=0; i<items.count();i++){
		wireless_scan *scan = &items[i];
		if(scan->b.essid_on){
			ApListItem *item = new ApListItem(scan);
			ap_list->insertItem(item, -1);
		}
	}
}

void MainWindow::clickConnect()
{
	printf("in %s\n",__func__);
}

void MainWindow::clickSettings()
{
	printf("in %s\n",__func__);
}

void MainWindow::resizeEvent(QResizeEvent *e)
{
	if(e->size().width() > e->size().height()){
		hbox->setDirection(QBoxLayout::LeftToRight);
		rvbox->setDirection(QBoxLayout::TopToBottom);
	}else{
		hbox->setDirection(QBoxLayout::TopToBottom);
		rvbox->setDirection(QBoxLayout::LeftToRight);
	}
	QWidget::resizeEvent(e);
	repaint();
}
