#include "scandialog.h"
#include "scanlist.h"
#include <qdialog.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qvbox.h>

ScanDialog::ScanDialog(ScanList *scan, QWidget *parent, const char *name)
	: QDialog(parent, name, true, WStyle_Customize | WStyle_NoBorderEx)
{
	resize(200,50);

	box = new QVBoxLayout(this);

	QVBox *vbox = new QVBox(this);
	vbox->setMargin(10);
	vbox->setSpacing(10);
	vbox->setFrameStyle(QFrame::WinPanel | QFrame::Raised);

	text = new QLabel(vbox);
	text->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);

	QHBox *hbox = new QHBox(vbox);
	cancel = new QPushButton(QString::fromUtf8("Cancel"), hbox);
	cancel->setFixedWidth(100);

	box->addWidget(vbox);

	connect(cancel, SIGNAL(clicked()),
			this, SLOT(clickCancel()));

	scanlist = scan;
}

ScanDialog::~ScanDialog()
{
}

int ScanDialog::exec()
{
	old_state = -1;
	startTimer(500);
	scanlist->start();
	return QDialog::exec();
}

void ScanDialog::timerEvent(QTimerEvent *e)
{
	QString tmp;
	int state = scanlist->getState();

	if(state == ScanList::SCAN_END){
		killTimers();
		scanlist->join();
		accept();
	}

	if(state == old_state)
		return;

	switch(state){
	case ScanList::SCAN_NONE:
		tmp = QString::fromUtf8("");
		break;
	case ScanList::SCAN_INIT:
		tmp = QString::fromUtf8("初期化中");
		break;
	case ScanList::SCAN_GET:
		tmp = QString::fromUtf8("スキャン中");
		break;
	case ScanList::SCAN_POST:
		cancel->setDisabled(true);
		tmp = QString::fromUtf8("停止中");
		break;
	case ScanList::SCAN_END:
		// never reached
		break;
	}
	text->setText(tmp);

	old_state = state;
}

void ScanDialog::clickCancel()
{
	scanlist->cancel();
	cancel->setDisabled(true);
}
